@@ -13,7 +13,7 @@
 ##############################################################################
 """Session implementation using cookies
 
-$Id: http.py 91714 2008-10-03 21:19:13Z jim $
+$Id: http.py 96645 2009-02-17 18:00:24Z nadako $
 """
 import hmac
 import logging
@@ -92,6 +92,23 @@ class ICookieClientIdManager(IClientIdManager):
             default=False,
             )
 
+    domain = schema.TextLine(
+            title=_('Effective domain'),
+            description=_(
+                "An identification cookie can be restricted to a specific domain "
+                "using this option. This option sets the ``domain`` attribute "
+                "for the cookie header. It is useful for setting one "
+                "identification cookie for multiple subdomains. So if this "
+                "option is set to ``.example.org``, the cookie will be available "
+                "for subdomains like ``yourname.example.org``. "
+                "Note that if you set this option to some domain, the identification "
+                "cookie won't be available for other domains, so, for example "
+                "you won't be able to login using the SessionCredentials plugin "
+                "via another domain."
+                ),
+            required=False,
+            )
+
     secure = schema.Bool(
         title=_('Request Secure communication'),
         required=False,
@@ -113,6 +130,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
     cookieLifetime = FieldProperty(ICookieClientIdManager['cookieLifetime'])
     secure = FieldProperty(ICookieClientIdManager['secure'])
     postOnly = FieldProperty(ICookieClientIdManager['postOnly'])
+    domain = FieldProperty(ICookieClientIdManager['domain'])
 
     def __init__(self):
         self.namespace = "zope3_cs_%x" % (int(time.time()) - 1000000000)
@@ -280,6 +298,14 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> bim.getRequestId(request) == bim.getRequestId(request2)
           True
 
+        Test a corner case where Python 2.6 hmac module does not allow
+        unicode as input:
+
+          >>> id_uni = unicode(bim.generateUniqueId())
+          >>> bim.setRequestId(request, id_uni)
+          >>> bim.getRequestId(request) == id_uni
+          True
+        
         If another server is managing the ClientId cookies (Apache, Nginx)
         we're returning their value without checking:
 
@@ -289,7 +315,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> request3._cookies = {'uid': 'AQAAf0Y4gjgAAAQ3AwMEAg=='}
           >>> bim.getRequestId(request3)
           'AQAAf0Y4gjgAAAQ3AwMEAg=='
-
+        
         """
         response_cookie = request.response.getCookie(self.namespace)
         if response_cookie:
@@ -309,8 +335,12 @@ class CookieClientIdManager(zope.location.Location, Persistent):
             if sid is None or len(sid) != 54:
                 return None
             s, mac = sid[:27], sid[27:]
-            if (digestEncode(hmac.new(s, self.secret, digestmod=sha).digest())
-                != mac):
+            
+            # call encode() on value s a workaround a bug where the hmac
+            # module only accepts str() types in Python 2.6
+            if (digestEncode(hmac.new(
+                    s.encode(), self.secret, digestmod=sha
+                ).digest()) != mac):
                 return None
             else:
                 return sid
@@ -383,6 +413,12 @@ class CookieClientIdManager(zope.location.Location, Persistent):
           >>> print request.response.getCookie(bim.namespace)
           {'path': '/', 'secure': True, 'value': '1234'}
 
+        If the domain is specified, it will be set as a cookie attribute.
+
+          >>> bim.domain = u'.example.org'
+          >>> bim.setRequestId(request, '1234')
+          >>> print request.response.getCookie(bim.namespace)
+          {'path': '/', 'domain': u'.example.org', 'secure': True, 'value': '1234'}
 
         When the cookie is set, cache headers are added to the
         response to try to prevent the cookie header from being cached:
@@ -422,6 +458,9 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         if self.secure:
             options['secure'] = True
 
+        if self.domain:
+            options['domain'] = self.domain
+
         response.setCookie(
             self.namespace, id,
             path=request.getApplicationURL(path_only=True),
