@@ -13,7 +13,7 @@
 ##############################################################################
 """Session tests
 
-$Id: tests.py 94410 2008-12-29 10:42:57Z icemac $
+$Id: tests.py 95586 2009-01-30 16:36:38Z thefunny42 $
 """
 from cStringIO import StringIO
 import unittest, os, os.path
@@ -64,7 +64,7 @@ class TestBootstrap(TestBootstrapSubscriber):
 
         import zope.component
         from zope.app.publication.zopepublication import ZopePublication
-        from zope.app.component.hooks import setSite
+        from zope.site.hooks import setSite
 
         cx = self.db.open()
         root = cx.root()
